﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.BusinessLogicsContracts;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using Microsoft.Extensions.Logging;

namespace PizzeriaBusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopLogic _shopLogic;
        private readonly IPizzaStorage _pizzaStorage;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopLogic shopLogic, IPizzaStorage pizzaStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _shopLogic = shopLogic;
            _pizzaStorage = pizzaStorage;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Incorrect Order status");
                return false;
            }

            model.Status = OrderStatus.Принят;

            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool UpdateStatus(OrderBindingModel model, OrderStatus newStatus)
        {
            CheckModel(model);

            if(model.Status + 1 != newStatus)
            {
                _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Incorrect Order status.");
                return false;
            }

            model.Status = newStatus;

            if(newStatus == OrderStatus.Выдан)
            {
                model.DateImplement = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
            }

            if (newStatus == OrderStatus.Готов)
            {
                var pizza = _pizzaStorage.GetElement(new PizzaSearchModel { Id = model.PizzaId });

                if(!TryRestoreShops(pizza, model.Count))
                {
                    _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Incorrect count.");
                    throw new ArgumentException("Количество изделий слишком велико");
                }
            }

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Выполняется);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Выдан);
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество пицц должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Sum:{ Sum}. Id: { Id}", model.Sum, model.Id);
        }

        private bool TryRestoreShops(IPizzaModel pizza, int count)
        {
            int freePlaces = 0;

            _shopLogic.ReadList(null).ForEach(x =>
            {
                int currentFreePlaces = x.Capacity;
                foreach (var pair in x.ShopPizzas)
                {
                    currentFreePlaces -= pair.Value.Item2;
                }
                freePlaces += currentFreePlaces;
            });

            if (freePlaces < count) return false;

            _shopLogic.ReadList(null).ForEach(x =>
            {
                if (count <= 0) return;

                int currentFreePlaces = x.Capacity;
                foreach (var elem in x.ShopPizzas)
                {
                    currentFreePlaces -= elem.Value.Item2;
                }

                _shopLogic.AddPizza(new ShopSearchModel { Id = x.Id }, pizza, Math.Min(currentFreePlaces, count));
                count -= currentFreePlaces;
            });

            return true;
        }

    }
}
